#![cfg_attr(
    all(not(debug_assertions), target_os = "windows"),
    windows_subsystem = "windows"
)]

mod picker;
mod rpc;

use paris::*;
use picker::led_color::LedColor;
use picker::manager::Picker;
use std::{
    time::Instant,
};

#[tauri::command]
async fn refresh_displays() {
    match Picker::global().refresh_displays().await {
        Ok(_) => {}
        Err(error) => {
            error!("{}", error)
        }
    }
}

#[tauri::command]
async fn take_snapshot() -> Vec<String> {
    let start = Instant::now();
    let manager = Picker::global();

    match manager.take_screenshots_for_all().await {
        Ok(screenshots) => {
            info!("screenshots len: {}", screenshots.len());
            let mut futures = Vec::new();
            for screenshot in screenshots {
                let future = screenshot.to_webp_base64().await;
                futures.push(future);
            }
            futures
        }
        Err(error) => {
            error!("can not take screenshots for all. {}", error);
            vec![]
        }
    }
}

#[tauri::command]
async fn get_led_strip_colors() -> Result<Vec<LedColor>, String> {
    let colors = Picker::global().get_led_strip_colors().await;
    match colors {
        Ok(colors) => {
            rpc::manager::Manager::global()
                .publish_led_colors(&colors)
                .await;
            Ok(colors)
        }
        Err(error) => Err(format!("{}", error)),
    }
}

async fn bitmap_to_webp_base64(width: usize, height: usize, bitmap: Vec<u8>) -> String {
    let mut bitflipped = Vec::with_capacity(width * height * 3);
    let stride = bitmap.len() / height;

    // let mut img = Image::new(w as u32, h as u32);
    for y in 0..height {
        for x in 0..width {
            let i = stride * y + 4 * x;
            bitflipped.extend_from_slice(&[bitmap[i + 2], bitmap[i + 1], bitmap[i]]);
            // img.set_pixel(
            //     x as u32,
            //     y as u32,
            //     Pixel::new(buffer[i + 2], buffer[i + 1], buffer[i]),
            // );
        }
    }

    let webp_memory =
        webp::Encoder::from_rgb(bitflipped.as_slice(), width as u32, height as u32).encode(100.0);
    return base64::encode(&*webp_memory);
}

#[tokio::main]
async fn main() {
    rpc::manager::Manager::global();
    tauri::Builder::default()
        .invoke_handler(tauri::generate_handler![
            take_snapshot,
            refresh_displays,
            get_led_strip_colors
        ])
        .run(tauri::generate_context!())
        .expect("error while running tauri application");
}
